@@ -118,7 +118,14 @@ class ManholeConnection(threading.Thread):
         pthread_setname_np(self.ident, "Manhole ----")
 
         client = self.client
-        client.settimeout(None)
+        del self.client
+        pid, _, _ = self.check_credentials(client)
+        pthread_setname_np(self.ident, "Manhole %s" % pid)
+
+        self.handle(client)
+
+    @staticmethod
+    def check_credentials(client):
         pid, uid, gid = get_peercred(client)
         euid = os.geteuid()
         client_name = "PID:%s UID:%s GID:%s" % (pid, uid, gid)
@@ -130,7 +137,11 @@ class ManholeConnection(threading.Thread):
             ))
 
         cry("Accepted connection %s from %s" % (client, client_name))
-        pthread_setname_np(self.ident, "Manhole %s" % pid)
+        return pid, uid, gid
+
+    @staticmethod
+    def handle(client):
+        client.settimeout(None)
         client.setsockopt(socket.SOL_SOCKET, socket.SO_SNDBUF, 0)
         client.setsockopt(socket.SOL_SOCKET, socket.SO_RCVBUF, 0)
         backup = []
@@ -170,7 +181,6 @@ class ManholeConnection(threading.Thread):
                     pass
                 del fh
             del junk
-            self.client = None
             sys.setcheckinterval(old_interval)
 
 def run_repl():
@@ -183,13 +193,23 @@ def run_repl():
         'traceback': traceback,
     }).interact()
 
+def _handle_oneshot(_signum, _frame):
+    try:
+        sock, pid = Manhole.get_socket()
+        cry("Waiting for new connection (in pid:%s) ..." % pid)
+        client, _ = sock.accept()
+        ManholeConnection.check_credentials(client)
+        ManholeConnection.handle(client)
+    except SystemExit:
+        pass
+
 def _remove_manhole_uds():
     name = "/tmp/manhole-%s" % os.getpid()
     if os.path.exists(name):
         os.unlink(name)
 
 _INST_LOCK = thread.allocate_lock()
-_STDERR = _INST = _CLIENT_INST = _ORIGINAL_OS_FORK = _ORIGINAL_OS_FORKPTY = None
+_STDERR = _INST = _CLIENT_INST = _ORIGINAL_OS_FORK = _ORIGINAL_OS_FORKPTY = _SHOULD_RESTART = None
 
 def _patched_fork():
     """Fork a child process."""
@@ -223,26 +243,35 @@ ALL_SIGNALS = [
     getattr(signal, sig) for sig in dir(signal)
     if sig.startswith('SIG') and '_' not in sig
 ]
-def install(verbose=True, patch_fork=True, activate_on=None, sigmask=ALL_SIGNALS):
-    global _STDERR, _INST, VERBOSE #pylint: disable=W0603
+def install(verbose=True, patch_fork=True, activate_on=None, sigmask=ALL_SIGNALS, oneshot_on=None):
+    global _STDERR, _INST, _SHOULD_RESTART, VERBOSE #pylint: disable=W0603
     with _INST_LOCK:
         VERBOSE = verbose
         _STDERR = sys.__stderr__
         if not _INST:
             _INST = Manhole(sigmask)
+            if oneshot_on is not None:
+                oneshot_on = getattr(signal, 'SIG'+oneshot_on) if isinstance(oneshot_on, basestring) else oneshot_on
+                signal.signal(oneshot_on, _handle_oneshot)
+
             if activate_on is None:
-                _INST.start()
+                if oneshot_on is None:
+                    _INST.start()
+                    _SHOULD_RESTART = True
             else:
-                signal.signal(
-                    getattr(signal, 'SIG'+activate_on) if isinstance(activate_on, basestring) else activate_on,
-                    _activate_on_signal
-                )
+                activate_on = getattr(signal, 'SIG'+activate_on) if isinstance(activate_on, basestring) else activate_on
+                if activate_on == oneshot_on:
+                    raise RuntimeError('You cannot do activation of the Manhole thread on the same signal that you want to do oneshot activation !')
+                signal.signal(activate_on, _activate_on_signal)
         atexit.register(_remove_manhole_uds)
         if patch_fork:
-            if activate_on is None:
+            if activate_on is None and oneshot_on is None:
                 _patch_os_fork_functions()
             else:
-                cry("Not patching os.fork and os.forkpty. Activation is done by signal %s" % activate_on)
+                if activate_on:
+                    cry("Not patching os.fork and os.forkpty. Activation is done by signal %s" % activate_on)
+                elif oneshot_on:
+                    cry("Not patching os.fork and os.forkpty. Oneshot activation is done by signal %s" % oneshot_on)
 
 def reinstall():
     global _INST #pylint: disable=W0603
@@ -250,7 +279,8 @@ def reinstall():
     with _INST_LOCK:
         if not _INST.is_alive():
             _INST = Manhole(_INST.sigmask)
-            _INST.start()
+            if _SHOULD_RESTART:
+                _INST.start()
 
 def dump_stacktraces():
     lines = []
@@ -270,7 +300,7 @@ def dump_stacktraces():
 if __name__ == '__main__': #pragma: no cover
     from logging import basicConfig, DEBUG
     basicConfig(level=DEBUG)
-    install(1)
+    install(verbose=True, oneshot_on='USR2')
     import faulthandler
     faulthandler.enable()
 
